declare
\insert timetabling/random

TTData=
data(weekDayNames : ['Monday' 'Tuesday' 'Wednesday'
		     'Thursday' 'Friday']
     earlyMorning : 9
     lateNight    : 13
     offTimes     : ['Monday'#10#10
		     'Wednesday'#10#10]
     numberOfRooms: 2
     students     : s(a:[101 104 201 202]
		      b:[101 105 204 205]
		      c:[102 103 201 202]
		      d:[105 107 203 204]
		      e:[101 104 202 203]
		      f:[102 103 201 202]
		      g:[104 105 201 202]
		      h:[103 106 201 202]
		     )
     courses      : cs(101: c(color: lightyellow1)
		       102: c(color: lightsalmon2)
		       103: c(color: cyan3)
		       104: c(color: lightgoldenrod1)
		       105: c(color: mistyrose3)
		       106: c(color: tomato)
		       107: c(color: lightyellow3)
		       201: c(color: red)
		       202: c(color: blue)
		       203: c(color: green)
		       204: c(color: yellow)
		       205: c(color: slategray)
		      )
     instructors  : is(a:i(preferences: ['Monday'#9#12]
			   courses    : [101 102 201])
		       b:i(preferences: ['Tuesday'#10#13]
			   courses    : [102 202])
		       c:i(preferences: ['Wednesday'#11#12]
			   courses    : [104 105 203])
		       d:i(preferences: ['Wednesday'#9#10]
			   courses    : [106 204])
		       e:i(preferences: ['Thursday'#9#12]
			   courses    : [107 205])
		      )
    )

declare
   NumberOfHoursPerDay= TTData.lateNight-TTData.earlyMorning+1
   WeekDays           = {MakeRecord abbr TTData.weekDayNames}
   {List.foldL TTData.weekDayNames
    fun {$ Acc I}
       WeekDays.I=Acc#(Acc+NumberOfHoursPerDay-1)
       Acc+NumberOfHoursPerDay
    end 0 _}
   BeginningOfWeek    = WeekDays.(TTData.weekDayNames.1).1
   EndOfWeek          = WeekDays.{List.last TTData.weekDayNames}.2

{Browse NumberOfHoursPerDay}
{Browse WeekDays}
{Browse BeginningOfWeek}
{Browse EndOfWeek}

declare
   fun {TimeToSpec Day#Start#End}
      WeekDays.Day.1 + Start - TTData.earlyMorning
      #
      WeekDays.Day.1 + End - TTData.earlyMorning
   end

{Browse {TimeToSpec 'Monday'#9#12}}

declare
   CourseHours = {Record.map TTData.courses
		  fun {$ C}
		     {FD.int BeginningOfWeek#EndOfWeek}
		  end}

{Browse CourseHours}

declare
   Quality
   = {FD.decl}
   = {FD.sum
      {Record.foldL
       TTData.instructors
       fun {$ Acc Instructor}
	  {FoldL Instructor.courses
	   fun {$ Acc Course}
	      {FD.reified.int
	       {Map Instructor.preferences TimeToSpec}
	       CourseHours.Course}
	      |Acc
	   end Acc}
       end nil}
      '=:'}

{Browse Quality}

in

declare
   Solution = root( courseHours: CourseHours
		    weekDays: WeekDays
		    quality: Quality)

{Browse Solution}

   %% Constraint: off times

   {ForAll TTData.offTimes
    proc {$ OffTime}
       {Record.forAll CourseHours
	proc {$ CourseHour}
	   CourseHour::compl({TimeToSpec OffTime})
	end}
    end}

   %% Constraint: No overlap for the
   %%             courses that a student takes

   {Record.forAll TTData.students
    proc {$ StudentCourses}
       {FD.distinct
	{Map StudentCourses
	 fun {$ C} CourseHours.C end}}
    end}

   %% Constraint: No Overlap for the
   %%             courses that an instructor teaches

   {Record.forAll TTData.instructors
    proc {$ Instructor}
       {FD.distinct
	{Map Instructor.courses
	 fun {$ C} CourseHours.C end}}
    end}

   %% Constraint: Number of available rooms is
   %%             not exceeded

   {For BeginningOfWeek EndOfWeek 1
    proc {$ I}
       {FD.atMost TTData.numberOfRooms CourseHours I}
    end}


   {FD.distribute
    generic(order:naive value:RandomValueSelection)
    CourseHours}
end


declare
proc {Ordering Old New}
   Old.quality <: New.quality
end

{Explorer script(Script Ordering)}

declare
\insert timetabling/gantt

{Explorer add(information TimetablingGantt)}

