%%%  Department of Information Systems and Computer Science
%%%  National University of Singapore
%%%  3101 Project: Flexible Timetabling
%%%  Students: Lee Mei Tien, Leo Hui Ting, Tay Choon Yee
%%%  Supervisor: A/P Joxan Jaffar
%%%  Submission Date: 6 October

%%%  File: graphics.oz
%%%  Description: routine for display of weekly timetable


proc {DrawSchedule Description Solution Parent}

   CanvasWidth   =720
   WeekDaysWidth =100
   RoomLimit     =Description.numberOfRooms
   HourWidth     =CanvasWidth div (Description.lateAtNight
				   - Description.earlyInTheMorning + 2)
   MorningOffset =HourWidth div 2
   TutorialHeight=20
   DayHeight     =(RoomLimit+1)*TutorialHeight
   TopOffset     =30
   TopNumOffset  =20
   CanvasHeight  ={Length Description.listOfWeekDays}*DayHeight+TopOffset
   
   XTutorialTextOffset=10
   YTutorialTextOffset=TutorialHeight div 2

   Font          =('-*-lucidatypewriter-*-*-*-*-'#
		   TutorialHeight#
		   '-*-*-*-*-*-*-*')

   CanvasColor = case Tk.isColor then mediumturquoise else white end

   {TimeTable save('CurrentOut.ozt')}

   %% window for schedule
   W = {New Tk.toplevel tkInit(parent:     Parent
			       background: CanvasColor
			       title: ' Tutorials ISCS ')} 

   %% canvas for the schedule
   Canvas = {New Tk.canvas tkInit(parent:W 
				  width:CanvasWidth
				  height:CanvasHeight)}

   %% canvas for the week days on the left
   WeekDays = {New Tk.canvas tkInit(parent:W
				    width:WeekDaysWidth
				    height:CanvasHeight)}

   {Tk.send pack(WeekDays Canvas side:left)}
   
in

   %% time line on top of window
   
   {Loop.for Description.earlyInTheMorning Description.lateAtNight+1 1
    proc{$ I}
       J=I-Description.earlyInTheMorning
       Text={Int.toString I}
       X=MorningOffset+J*HourWidth
    in 
       {Canvas tk(crea line X 0 X 10)}
       {Canvas tk(crea text X TopNumOffset text:Text)} 
    end}

   %% all the rest
   
   {List.forAllInd Description.listOfWeekDays
    proc{$ Ind Day}
       %% put the weekday in the weekday canvas
       {WeekDays tk(crea text 5 Ind*DayHeight-(DayHeight div 2)+TopOffset
		    text:Day anchor:w)}
       %% make a line where the day starts
       YStartOfDay=TopOffset+(Ind-1)*DayHeight
       {Canvas tk(crea line 0 YStartOfDay
		  CanvasWidth YStartOfDay)}
       
       TutorialsOnThatDay = {Record.foldRInd Solution.tutorials
			     fun {$ I X Acc}
				{Append
				 {Record.foldRInd
				  X
				  fun {$ J T Acc}
				     %% We know that start time is fixed!
				     thread 
					if {IsDet T true}
					   T::(Solution.weekDays.Day)
					then
					   tutorial(lecture:I
						    tutorialId:J
						    start:T
						    hour:T-Solution
						    .weekDays.Day.1)
					   |Acc
					else
					   Acc
					end
				     end
				  end nil}
				 Acc}
			     end nil}

       SortedTutorials = {Sort TutorialsOnThatDay
			  fun {$ L1 L2}
			     L1.start < L2.start
			     orelse
			     (L1.start == L2.start
			      andthen
			      (L1.lecture < L2.lecture
			       orelse
			       (L1.lecture == L2.lecture
				andthen
				L1.tutorialId < L2.tutorialId)))
			  end}
    in
       %% display all tutorials of that day
       {List.foldL SortedTutorials
	fun {$ Row#Hour tutorial(lecture:I tutorialId:J start:T hour:NewHour)}
	   NewRow
	   case Hour==NewHour then
	      NewRow = Row + 1
	   else
	      NewRow = 0
	   end
	   XStart=MorningOffset+NewHour*HourWidth
	   YStart=YStartOfDay+NewRow*TutorialHeight
	in
	   {Canvas tk(crea rectangle XStart YStart
		      XStart+HourWidth YStart+TutorialHeight
		      fill:Description.colors.I)}
	   {Canvas tk(crea text
		      XStart+XTutorialTextOffset
		      YStart+YTutorialTextOffset
		      text:I#'.'#J
		      anchor:w
		      font:Font)}
	   NewRow#NewHour
	end
	0#~1 _}
    end}
end
