proc {TimetablingScript Solution}
   NumberOfHoursPerDay = TTData.lateNight-TTData.earlyMorning+1

   WeekDays            = {MakeRecord abbr TTData.weekDayNames}

   {List.foldL TTData.weekDayNames
    fun {$ Acc I}
       WeekDays.I=Acc#(Acc+NumberOfHoursPerDay-1)
       Acc+NumberOfHoursPerDay
    end 0 _}

   BeginningOfWeek     = WeekDays.(TTData.weekDayNames.1).1

   EndOfWeek           = WeekDays.{List.last TTData.weekDayNames}.2

   fun {TimeToSpec Day#Start#End}
      WeekDays.Day.1 + Start - TTData.earlyMorning
      #
      WeekDays.Day.1 + End - TTData.earlyMorning
   end

   CourseHours = {Record.map TTData.courses
		  fun {$ C}
		     {FD.int BeginningOfWeek#EndOfWeek}
		  end}
   Quality
   = {FD.decl}
   = {FD.sum
      {Record.foldL
       TTData.instructors
       fun {$ Acc Instructor}
	  {FoldL Instructor.courses
	   fun {$ Acc Course}
	      {FD.reified.int
	       {Map Instructor.preferences TimeToSpec}
	       CourseHours.Course}
	      |Acc
	   end Acc}
       end nil}
      '=:'}

in

   Solution = root( courseHours: CourseHours
		    weekDays: WeekDays
		    quality: Quality)

   %% Constraint: off times

   {ForAll TTData.offTimes
    proc {$ OffTime}
       {Record.forAll CourseHours
	proc {$ CourseHour}
	   CourseHour::compl({TimeToSpec OffTime})
	end}
    end}

   
   %% Constraint: No overlap for the
   %%             courses that a student takes
   
   {Record.forAll TTData.students
    proc {$ StudentCourses}
       {FD.distinct
	{Map StudentCourses
	 fun {$ C} CourseHours.C end}}
    end}

   %% Constraint: No Overlap for the
   %%             courses that an instructor teaches

   {Record.forAll TTData.instructors
    proc {$ Instructor}
       {FD.distinct
	{Map Instructor.courses
	 fun {$ C} CourseHours.C end}}
    end}
   
   %% Constraint: Number of available rooms is
   %%             not exceeded

   {For BeginningOfWeek EndOfWeek 1
    proc {$ I}
       {FD.atMost TTData.numberOfRooms CourseHours I}
    end}

   %% distribute course->hour assignment
   {FD.distribute naive CourseHours}
end
