%%% Copyright (c) 1998
%%% School of Computing
%%% National University of Singapore
%%% Project: InteGrate

%%% Last modified: $Date: 1998/11/09 13:55:45 $ by $Author: huyl $
%%% Version: $Revision: 1.15 $

%%%  Description: Auxiliary code for tcl/tk interface;
%%%               used in gui.oz, gantt.oz, airport.oz


proc {TkFlush Window}
   {Wait {Tk.return winfo(exists Window)}}
end

fun {Exist Window}
   {Tk.returnInt winfo(exists Window)}
end

proc{Forget GM Window}
   case {Exist Window}==1  
   then {Tk.send GM(forget Window)}
   else skip end
end

class FileName from BaseObject
   attr fileName: unit
      entry:unit
   meth init(Entry)
      entry<-Entry
   end
   meth get($)
      case @fileName==unit
      then
	 {Error 'choose file first'}
	 raise nofile end
      else
	 @fileName
      end
   end
   meth put(F)
      fileName <- F
      {@entry tk(delete 0 'end')}
      {@entry tk(insert(0 F))}
      {@entry tk(xview moveto 1.0)}
   end
end

proc {Skip} skip end

%% error routine
proc {Error VS}
\ifdef NOX
   {Show VS}
\else
   {New TkTools.error tkInit(text:VS) _}
\endif
end

proc {WaitGround R}
   case {IsRecord R}
   then
      {Record.forAll R proc {$ X} {Wait X} end}
   else
      {Wait R}
   end
end

proc {CreatScrollbar Parent Orient ?ScrollBar}
   ScrollBar = {New Tk.scrollbar
		tkInit(parent: Parent
		       relief: sunken
		       bd:     Border
		       width:  ScrollerWidth
		       orient: Orient)}
end

local
   proc {MakeScrollbar Canvases Frame Orient ?Scrollbar}
      View ScrollCommand
      case Orient
      of horizontal then View=xview ScrollCommand=xscrollcommand
      [] vertical then View=yview ScrollCommand=yscrollcommand
      end
      
      proc {Handler Mess}
	 case Mess
	 of m("moveto" F) then
	    {ForAll Canvases proc {$ Canvas}
				{Canvas tk(View 'moveto' F)}
			     end}
	 [] m("scroll" N What) then
	    {ForAll Canvases proc {$ Canvas}
				{Canvas tk(View 'scroll' N What)}
			     end}
	 end
      end
   in
      Scrollbar = {New Tk.scrollbar tkInit(parent: Frame
					   relief: sunken
					   bd:     Border
					   width:  ScrollerWidth
					   orient: Orient
					   action: Handler # m)}
  
      {Tk.batch {Map Canvases
		 fun {$ Canvas}
		    o(Canvas configure ScrollCommand:s(Scrollbar set))
		 end}}
   end
in   
   proc {MakeXScrollbar Canvases Frame ?ScrX}
      {MakeScrollbar Canvases Frame horizontal ?ScrX}
   end

   proc {MakeYScrollbar Canvases Frame ?ScrY}
      {MakeScrollbar Canvases Frame vertical ?ScrY}
   end
end

proc {Pack Frame Spec}
   case {IsList Spec}
   then {ForAll Spec proc {$ X} {Pack Frame X} end}
   else
      case Spec of S#W
      then
	 L={Label S}
	 SInit
      in
	 case {Member L [top bottom left right grid]}
	 then
	    W={New Tk.frame tkInit(parent:Frame)}
	    case L==grid then
	       {Tk.send pack(W side:left fill:both padx:2 pady:2)}
	       {Grid W S.1}
	    else
	       {Tk.send
		case {Width S} == 1
		then pack(W side:L fill:y padx:2 pady:2)
		elsecase {Width S} == 2
		then
		   {Adjoin
		    pack(W side:L fill:y padx:2 pady:2)
		    S.2}
		end} %% end of {Tk.send...
	       {Pack W S.1}
	    end %% end of case L==grid 
	 else
	    case {HasFeature S pack}
	    then
	       SInit={Record.subtract S pack}
	       {Tk.send
		{Adjoin
		 pack(W={New DefaultWidgetSpec.L.library.L
			 {Adjoin DefaultWidgetSpec.L.spec
			  {Adjoin SInit tkInit(parent:Frame)}}}
		     )
		 S.pack}}
	    else
	        {Tk.send
		 pack(W={New DefaultWidgetSpec.L.library.L
			 {Adjoin DefaultWidgetSpec.L.spec
			  {Adjoin S tkInit(parent:Frame)}}})}
	    end
	 end
      else
	 {Pack Frame Spec#_}
      end
   end
end

proc {Grid Frame Spec}
   {Record.forAllInd Spec
    proc {$ RI RowSpec}
       {Record.forAllInd RowSpec
	proc {$ CI ColumnSpec}
	   proc {Do ColumnSpec}
	      %{Show asdf#ColumnSpec}
	      case ColumnSpec of S#W
	      then
		 L={Label S}
	      in
		 case {Member L [s n e w ns we nsw nse nswe]}
		 then
		    GridSpec=case {Width S} of 1 then grid else S.2 end
		    L1={Label S.1}
		 in
		    {Tk.send
		     {Adjoin GridSpec
		     grid(W={New DefaultWidgetSpec.L1.library.L1
			     {Adjoin DefaultWidgetSpec.L1.spec
			      {Adjoin S.1 tkInit(parent:Frame)}}
			    }
			  row: RI
			  column: CI
			  sticky: L)}}
		 else {Do w(S)#W}
		 end
	      else {Do ColumnSpec#_}
	      end
	   end
	in
	   {Do ColumnSpec}
	end}
    end}
end

proc {PackTextFrame Text Frame Spec}
   NewTextFrame = {New TkTools.textframe
		   tkInit(parent:Frame
			  text:Text
			  font:NormalFont)}
in
   {Tk.send pack(NewTextFrame side:top fill:x padx:5 expand:1)}
   {Pack NewTextFrame.inner Spec}
end

DefaultWidgetSpec
=
spec(label: spec(library:Tk
		 spec: tkInit(
			      font:NormalFont
			      bd:2
			      text:""
			     )
		)
     entry:       spec(library:Tk
		       spec:tkInit(
				   font:NormalFont
				   highlightthickness:0
				   bd:2
				   relief:sunken
				   state:normal
				  )
		      )
     radiobutton: spec(library:Tk
		       spec:tkInit(
				   bd:2
				   font:NormalFont
				   highlightthickness:0
				   action:Skip
				  )
		      )
     checkbutton: spec(library:Tk
		       spec:tkInit(
				   bd:2
				   font:NormalFont
				   highlightthickness:0
				   action:Skip
				  )
		      )
     button: spec(library:Tk
		  spec:tkInit(
			      bd:2
			      font:NormalFont
			      highlightthickness:0
			      action:Skip
			     )
		 )
     scale: spec(library:Tk
		 spec:tkInit(
			     font:NormalFont
			     orient:horizontal
			    )
		)
     canvas:spec(library:Tk
		 spec:tkInit
		)
     frame:spec(library:Tk
		spec:tkInit
	       )
     listbox:spec(library:Tk
		  spec:tkInit
		 )
     scrollbar:spec(library:Tk
		    spec:tkInit
		   )
     message:spec(library:Tk
		  spec:tkInit
		 )

     numberentry:spec(library:TkTools
		      spec:tkInit
		     )
     note:spec(library:TkTools
	       spec:tkInit
	      )
     notebook:spec(library:TkTools
		   spec:tkInit
		  )
     textframe:spec(library:TkTools
		    spec:tkInit
		   )
     menubar:spec(library:TkTools
		  spec:tkInit
		 )
     dialog:spec(library:TkTools
		  spec:tkInit
		 )
     
    )


class ScalingCanvas from Tk.canvas
   attr
      scale: AirportDefScale
      xFraction:0.0
      yFraction:0.0
      width
      height

   /*   
   meth xview(A B)
      case A=='moveto' then
	 xFraction <- case B
		      of "0" then 0.
		      else {String.toFloat
			    case B.1==&- then &~|B.2 else B end}
		      end
	
      else skip
      end
      %Tk.canvas,tk(xview A B)
      Tk.canvas,tk(xview moveto @xFraction)
   end

   meth yview(A B)
      case A=='moveto' then
	 yFraction <- case B
		      of "0"  then 0.
		      else {String.toFloat
			    case B.1==&- then &~|B.2 else B end}
		      end

      else skip
      end
      %Tk.canvas,tk(yview A B)
      Tk.canvas,tk(yview moveto @yFraction)
   end
   */
   
   meth init
      Tk.canvas,tkBind(event: '<Configure>'
		       action:self#Resized
		       args: [float(h) float(w)]
		       append:true)
   end
   
   meth Resized(H W)
      height <- H
      width  <- W
   end
   
   meth scale(Scale)
      ScaleBy=Scale/@scale
   in
      scale<-Scale
      ScalingCanvas, adjust
      Tk.canvas, tk(scale all 0 0 ScaleBy ScaleBy)
      {Tk.batch [
		 o(self xview moveto @xFraction)
		 o(self yview moveto @yFraction)
		]}
      
   end
   
   meth adjust
     
      %%first to get the middle position of the current view of canvas
      [XS1 XS2] = {Tk.returnList o(self xview)}
      XF1=case XS1 
	  of "0" then 0.
	  else {String.toFloat case XS1.1==&- then &~|XS1.2 else XS1 end}
	  end
      XF2 = case XS2 of "1" then 1. else {String.toFloat XS2} end
      XFMid=(XF1+XF2)/2.0 
      
      [YS1 YS2]={Tk.returnList o(self yview)}
      YF1=case YS1 
	  of "0" then 0.
	  else {String.toFloat case YS1.1==&- then &~|YS1.2 else YS1 end}
	  end
      YF2 = case YS2 of "1" then 1. else {String.toFloat YS2} end
      YFMid=(YF1+YF2)/2.0

      %%next to get the range of the canvas after scaling
      Scale=@scale
      Left   = 0.0
      Right  = Scale  * AirportXMaxFloat
      Top    = 0.0 
      Bottom = Scale  * AirportYMaxFloat

      %ScrInc={FloatToInt {IntToFloat DefScrInc}*Scale}
      
   in
      %% next try to keep the middle position as the center of scaling
      xFraction <-  XFMid-(@width/2.0)/(Right-Left)
      yFraction  <- YFMid-(@height/2.0)/(Bottom-Top)
   
      {Tk.batch [
		 o(self configure scrollregion:q(Left Top Right Bottom))
		% o(self configure xscrollincrement:ScrInc)
		% o(self configure yscrollincrement:ScrInc)
		]}
   end
   
end

